#!/usr/bin/env node
"use strict";

const path = require("path");
const chalk = require("chalk");
const program = require("commander");

const { version } = require("../package");
const getDefaults = require("./defaults");
const prompts = require("./prompt");
const create = require("./create");

(async () => {
  const defaults = await getDefaults();

  program
    .name("create-noah-extension")
    .version(version)
    .usage("[options] [extension-name]")
    .option("-d, --desc <string>", "package description")
    .option("-a, --author <string>", "author's github handle", defaults.author)
    .option("-g, --no-git", "generate without git init")
    .option(
      "-m, --manager <npm|yarn>",
      "package manager to use",
      /^(npm|yarn)$/,
      defaults.manager
    )
    .option(
      "-t, --template <default>",
      "package template to use",
      /^(default)$/,
      defaults.template
    )
    .option(
      "-s, --skip-prompts",
      "skip all prompts (must provide extension-name via cli)"
    )
    .parse(process.argv);

  const opts = {
    description: program.desc,
    author: program.author,
    manager: program.manager,
    template: program.template,
    skipPrompts: program.skipPrompts,
    git: program.git
  };

  Object.keys(opts).forEach(key => {
    if (!opts[key] && defaults[key]) {
      opts[key] = defaults[key];
    }
  });

  if (program.args.length === 1) {
    opts.name = program.args[0];
  } else if (program.args.length > 1) {
    console.error("invalid arguments");
    program.help();
    process.exit(1);
  }

  const params = await prompts(opts);
  const dest = await create(params);

  console.log(`
  Your extension has been created at ${dest}.

  To get started, in one tab, run:
  $ ${chalk.cyan(`cd ${params.shortName} && ${params.manager} start`)}
  `);
})();
