"use strict";

const handlebars = require("handlebars");
const execa = require("execa");
const fs = require("fs-extra");
const globby = require("globby");
const ora = require("ora");
const path = require("path");
const pEachSeries = require("p-each-series");
const uuid = require('uuid')

const pkg = require("../package");

const templateBlacklist = new Set([]);

module.exports = async info => {
  const { manager, template, name, templatePath, git } = info;

  // handle scoped package names
  const parts = name.split("/");
  info.shortName = parts[parts.length - 1];

  const dest = path.join(process.cwd(), info.shortName);
  info.dest = dest;

  if (fs.existsSync(dest)) {
    console.log(
      `${dest} is not empty! Please remove it or change package name.`
    );
    process.exit(0);
  }

  await fs.mkdir(dest);

  const source = path.join(__dirname, "..", "templates", template);

  const copy = fs.copy(source, dest);

  ora.promise(copy, `Copying ${template} template to ${dest}`);

  await copy;

  const pkg = path.join(dest, "package.json")
  info.uuid = uuid.v1()

  const hbs = handlebars.compile(fs.readFileSync(pkg, "utf8"));
  const content = hbs({
    ...info,
    yarn: info.manager === "yarn"
  });

  fs.writeFileSync(pkg, content, "utf8");

  const install = execa(manager, ["install"], {
    cwd: dest,
    stdio: "inherit"
  });

  // install.stdout.pipe(process.stdout);

  console.log(`Running ${manager} install`);

  await install;

  if (git) {
    const git = initGitRepo({ dest });
    ora.promise(git, "Initializing git repo");
    await git;
  }

  return dest;
};

async function initGitRepo(opts) {
  const { dest } = opts;

  const gitIgnorePath = path.join(dest, ".gitignore");
  fs.writeFileSync(
    gitIgnorePath,
    `
# See https://help.github.com/ignore-files/ for more about ignoring files.

# dependencies
node_modules

# builds
build
dist
.rpt2_cache

# misc
.DS_Store
.env
.env.local
.env.development.local
.env.test.local
.env.production.local

npm-debug.log*
yarn-debug.log*
yarn-error.log*
`,
    "utf8"
  );

  const commands = [
    {
      cmd: ["init", "."],
      cwd: dest
    },
    {
      cmd: ["add", "."],
      cwd: dest
    },
    {
      cmd: ["commit", "-m", `"init ${pkg.name}@${pkg.version}"`],
      cwd: dest
    }
  ];

  return pEachSeries(commands, async ({ cmd, cwd }) => {
    return execa("git", cmd, { cwd: cwd });
  });
}
