"use strict";

const path = require("path");
const fs = require("fs");
const inquirer = require("inquirer");
const validateNpmName = require("validate-npm-package-name");

module.exports = async opts => {
  if (opts.name && !validateNpmName(opts.name).validForNewPackages) {
    throw new Error(`invalid package name "${opts.name}"`);
  }

  if (opts.skipPrompts) {
    if (!opts.name) {
      throw new Error(
        "invalid input; you must pass a package name with --skip-prompts"
      );
    }

    Object.keys(opts).forEach(key => {
      const value = opts[key];
      if (typeof value === "function") {
        opts[key] = value(opts);
      }
    });

    return opts;
  } else {
    const info = await inquirer.prompt([
      {
        type: "input",
        name: "name",
        message: "Extension Name",
        validate: name => {
          return name && validateNpmName(name).validForNewPackages;
        },
        default: opts.name
      },
      {
        type: "input",
        name: "description",
        message: "Package Description",
        default: opts.description
      },
      {
        type: "input",
        name: "author",
        message: "Author's GitHub Handle",
        default: opts.author
      },
      {
        type: "list",
        name: "manager",
        message: "Package Manager",
        choices: ["npm", "yarn"],
        default: opts.manager
      },
      {
        type: "list",
        name: "template",
        message: "Template",
        choices: ["default"],
        default: opts.template
      }
    ]);

    return {
      ...info,
      git: opts.git
    };
  }
};
